/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility");

dojo.require("dojox.data.dom");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

/* This class is intended to be a superclass only.  The subclass must implement a method called "_getChanges"
 * which returns an object of the form:
 * {
 *   "changes": [
 *     {
 *       "name": "com.ibm.team.property.name",
 *       "value": "newValue",
 *       "serviceImplementationName": "com.ibm.team.implementation.class.name"
 *     },
 *     {
 *       ...
 *     }
 *   ],
 *   "properties": [
 *     ServerPropertyElement1,
 *     ServerPropertyElement2,
 *     ...
 *   ]
 * }
 */

(function() {

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility", null, {
	
	_createSaveHandler: function (saveButton, errMsg) {
		this._saveErrMsg = errMsg;
		dojo.connect(saveButton, "onclick", this, "_saveHandler");
	},
	
	_saveHandler: function () {
		var invokedSave = false;
		var validation = this._validateChanges(this._getChanges().properties);
		if (validation === true) {
			var allChanges = this._getChanges();
			return this._saveChanges(allChanges);
		}
		else if (validation == "pending") {
			setTimeout(dojo.hitch(this, "_saveHandler"), 250);
		}
		else {
			if (this._saveErrMsg != null) {
				this._displayErrorMessage(this._saveErrMsg);
			}
			else {
				this._displayErrorMessage("There are some properties with invalid values. Please correct any properties flagged below and attempt to save again");
			}
		}
		return invokedSave;
	},
	
	_validateChanges: function (properties) {
		for (var i = 0; i < properties.length; i++) {
			if (properties[i]._validationPending) {
				return "pending";
			}
			else if (properties[i]._invalid) {
				return false;
			}
		}
		return true;
	},
	
	_saveChanges: function (allChanges) {
		if (allChanges != null && allChanges["changes"] != null && allChanges["changes"].length > 0) {
			this._pendingChanges = allChanges["properties"];
			var changeArgs = {
				updates: dojo.toJson(allChanges["changes"])
			};
			var srh = new ServiceResponseHandler(this, "_submitConfigCallback", "_submitConfigError");
			ServerStatusClient.postConfigurationUpdates(srh, changeArgs);
			return true;
		}
		return false;
	},
		
	_submitConfigCallback: function () {
		var serverRestart = false;
		for (var i = 0; i < this._pendingChanges.length; i++) {
			if (this._pendingChanges[i]._restartRequired) {
				serverRestart = true;
			}
			this._pendingChanges[i]._handleSuccessfulChange();
		}
		this._saveObj = null;
		var message = "Configuration changes saved successfully.";
		if (serverRestart) {
			message += "  However, some of your changes will require a server restart before they take effect.";
		}
		this._displaySuccessMessage(message);
	},
	
	_submitConfigError: function () {
		this._displayErrorMessage("An error occurred while saving the configuration properties.  Please check your server log for more details");
	},

	_displayErrorMessage: function(message, errorObj) {
		if (errorObj) {
			message += " (detailed message: " + 
				(errorObj.status === 0 ? "server unreachable - check network connectivity or contact server admin" : errorObj.message) +
				").";
		}
		this._displayMessage(message, "errorMsg");
	},
	
	_displaySuccessMessage: function (message) {
		this._displayMessage(message, "successMsg");
	},
	
	// do not call directly
	_displayMessage: function(message, className) {
		if (this.messageArea) {
			dojox.data.dom.removeChildren(this.messageArea);
			this.messageArea.className= "display " + className;
			this.messageArea.appendChild(document.createTextNode(message));
		}
	}

});

})();