/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.LdapRealm");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");

(function() {

var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerPropertyService = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyService;
var ConfigUtility = com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility;
var ServerPropertyServiceRow = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyServiceRow;
var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;

var PACKAGE_PREFIX = "com.ibm.team.repository.web.ui.internal.wizard.";
//var LDAP_SERVICE_NAME = "com.ibm.team.repository.service.internal.userregistry.LDAPUserRegistryProvider";
var LDAP_SERVICE_NAME = "com.ibm.team.repository.service.internal.userregistry.ldap.LDAPUserRegistryProvider";

var DISPLAY_PROPS = [];
DISPLAY_PROPS.LDAP1 = [
	"ldap.registryLocation",
	"ldap.registryUserName", 
	"ldap.registryPassword"
];
DISPLAY_PROPS.LDAP2 = [
	"ldap.baseUserDN",
	"ldap.userAttributesMapping"
];
DISPLAY_PROPS.LDAP3 = [
	"ldap.baseGroupDN", 
	"ldap.groupMapping", 
	"ldap.groupNameAttribute",
	"ldap.membersOfGroup"
];

/*          	TEST/REVERT/NEXT/EDIT
LOADING        		N   N   N   N
TESTING        		N   N   N   N
PENDING_OK 	      	Y   ?   N   Y 
PENDING_ERR 	    N   Y   N   Y
PENDING_WARN  	    Y   Y   N   Y
CONNECTED_OK        N   Y   Y   Y
CONNECTED_ERR       N   Y   N   Y
CONNECTED_WARN      N   Y   Y   Y  
*/
var STATUS = {
	LOADING: {test: false, revert: false, next: false, edit: false, code: "PROGRESS",
		message: "Loading LDAP settings..."},  
	TESTING: {test: false, revert: false, next: false, edit: false, code: "PROGRESS", 
		message: "Testing the conection to the LDAP server..."},  
	PENDING_OK: {test: true, revert: true, next: false, edit: true, code: "INFO", 
		message: "The configuration has pending changes.  Please test the connection to continue."},  
	PENDING_ERR: {test: false, revert: true, next: false, edit: true, code: "ERROR",
		message: "Some property values are invalid.  Please correct flagged properties to continue."}, 
	PENDING_WARN: {test: true, revert: true, next: false, edit: true, code: "WARNING",
		message: "Please verify properties flagged with warnings.  Test the connection to continue."}, 
	CONNECTED_OK: {test: true, revert: true, next: true, edit: true, code: "OK",  
		message: "LDAP connection is established.  Click Next to continue."},
	CONNECTED_ERR: {test: true, revert: true, next: false, edit: true, code: "ERROR",
		message: "Testing the LDAP configuration resulted in errors.  Please verify settings and retest."}, 
	CONNECTED_WARN: {test: true, revert: true, next: true, edit: true, code: "WARNING",
		message: "LDAP connection was attempted with warnings.  Resolve warnings or click Next to continue."}, 
	SAVE_ERR: {test: true, revert: true, next: false, edit: true, code: "ERROR",
		message: "An error occurred while saving the configuration properties.  Please check your server log for more details."}  
};

dojo.declare(PACKAGE_PREFIX + "LdapRealm",  [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/SetupLdapDetail.html"),
	heading: "Configure Jazz Team Server to use an LDAP server to act as my user registry",
	description: "Once you have configured your application server and LDAP server as described in the server setup guide, complete this form to configure the Jazz Team Server to use your LDAP server for user and group information.",
	userDescription: "Specify the following properties to configure Jazz Team Server to map to users in the LDAP server.",
	groupDescription: "Specify the following properties to configure Jazz Team Server to map to groups in the LDAP server.",
	computeDescription: "Based on the configuration values provided above, we can compute the remaining LDAP queries.  It is recommended that you select the option below to compute the additional queries.  You can view and modify the computed settings at any time from the Admin Configuration UI.",
	computeLabel: "Compute and save additional LDAP queries based on these configuration values",

	postCreate: function() {
		com.ibm.team.repository.web.ui.internal.wizard.LdapRealm.superclass.postCreate.apply(this);
		this.setStatusCallback({type: "PROGRESS", message: "Loading LDAP settings..."});
		this._initializeContent();
	},
		
	// INITAL LOAD METHODS
	_initializeContent: function () {
		var serviceConfigProps = {
			serviceInfo: {
				implementationClassName: LDAP_SERVICE_NAME
			},
			enableEdits: true,
			displayIn: this._hiddenTableBody,
			descriptionMode: "table", 
			initialMode: "edit"
		}
		this.ldapConfig = new ServerPropertyService(serviceConfigProps);
		dojo.connect(this.ldapConfig, "_markChanged", this, function() {
			this._setState(STATUS.PENDING_OK);
		});
		dojo.connect(this.ldapConfig, "_checkUnchanged", this, function () {
				if(this.ldapConfig._changed) {
					this._setState(STATUS.PENDING_OK);
				} else {
					this._setState(this._initialLoadedState);
				}
			});
		dojo.connect(this.ldapConfig, "_markInvalid", this, "_handleInvalidInput");
		dojo.connect(this.ldapConfig, "_show", this, "_initializePropertiesAfterLoaded");
	},
	
	_initializePropertiesAfterLoaded: function () {
		this._initializePropertyDisplay(this._ldapEditPropertiesBody, DISPLAY_PROPS.LDAP1);
		this._initializePropertyDisplay(this._ldapUserEditPropertiesBody, DISPLAY_PROPS.LDAP2);
		this._initializePropertyDisplay(this._ldapGroupEditPropertiesBody, DISPLAY_PROPS.LDAP3);

		this._testConnection();
	},	
	
	_initializePropertyDisplay: function (attachPoint, displayProps) {	
		var targetPropertyServiceRow = new ServerPropertyServiceRow({
			_enableToggle: false,
			rowClass: ["indentNone editProperties"],
			_descriptionMode: "table"
		});
		attachPoint.appendChild(targetPropertyServiceRow.domNode);
				
		var longExists = false;
		var shortExists = false;
		for (var i=0; i < displayProps.length; i++) {
			var propertyElement = this.ldapConfig.getServerPropertyElement(displayProps[i]);
			if (propertyElement.elementInfo.propertyType === "STRING") {
				longExists = true;
				var table = targetPropertyServiceRow._longPropertiesTableBody;
			} else {
				shortExists = true;
				var table = targetPropertyServiceRow._shortPropertiesTableBody;
			}
			table.appendChild(propertyElement.domNode);
		};
		if (!longExists) {
			targetPropertyServiceRow._longPropertiesTableHeader.style.display = "none";
		}
		if (!shortExists) {
			targetPropertyServiceRow._shortPropertiesTableHeader.style.display = "none";
		}
	},
		
	// HANDLE USER INTERACTION METHODS
	 onInnerButtonAreaClick: function(e) {
		var button = e.target;
		if (button === this._revertButton) {
			this._revert();
		} else if (button === this._testConnectionButton){
			this._testConnection();
		} 
	}, 
		
	_handleInvalidInput: function (msg){
		if (msg.code === "ERROR") {
			this._setState(STATUS.PENDING_ERR);
		} else {
			this._setState(STATUS.PENDING_WARN);
		}
	}, 
	
	// TEST CONNECTION METHODS
	_testConnection: function() {
		this._setState(STATUS.TESTING);
		this.ldapConfig.testRobustValidity(new SRH(this, "_testConnectionCallback", "_testConnectionError"));
	},
	
	_testConnectionCallback: function(msg) {
		var connected = null;
		var message = "";
		if (msg != null) {
			connected = STATUS.CONNECTED_WARN;
			for (var i = 0; i < msg.length; i++) {
				if (msg[i].code === "ERROR") {
					connected = STATUS.CONNECTED_ERR;
				}
				message = message + msg[i].message + "  <br>";
			}
		} else {
			connected = STATUS.CONNECTED_OK;
		}
		if (this._initialLoadedState == null) {
			this._initialLoadedState = connected;
		}
		this._setState(connected, message);
	},
	
	_testConnectionError: function(errorObj) {
		this._setState(STATUS.CONNECTED_ERR, "An error occurred while fetching connection status.");
	},
	
	_revert: function() {
		this.ldapConfig.restoreSaved();
		this._setState(this._initialLoadedState);
	},

	// HANDLE STATE METHODS
	_setState: function(state, optionalMsg) {
		// check required for initial load before service properties are loaded
		if (this.ldapConfig != null) {
			this._allowPropertyEdit(state.edit); 
		}	
		var enableRevert = state.revert;
		if (enableRevert && !this.ldapConfig._changed) {
			enableRevert = false;
		}
		this._revertButton.disabled = (!enableRevert);
		this._testConnectionButton.disabled = (!state.test);

		var statusMessage = optionalMsg != null ? state.message + "<br>" + optionalMsg  : state.message;
		var statusCallback = {type: state.code, message: statusMessage, overrideNext: state.next};
		this._lastStatusCallback = statusCallback;
		this.setStatusCallback(statusCallback);
		this._setStatusIndication(state.code, state.message)
	},
	
	_setStatusIndication: function (code, message) {
		if (code != "INFO") {
			var indication = document.createElement("div");
			indication.className = "stepStatus statusMsg " + code;
			indication.innerHTML = message;

			dojox.data.dom.removeChildren(this.statusIndicationArea);
			dojo.toggleClass(this.statusIndicationArea, "noDisplay", false);
			this.statusIndicationArea.appendChild(indication);
			if (code != "PROGRESS") {
				setTimeout(dojo.hitch(this, function(){
					dojo.toggleClass(this.statusIndicationArea, "noDisplay", true);
				}), 3500);
			}
		}
	},

	
	validate: function () {
		if (this._lastStatusCallback) {
			this.setStatusCallback(this._lastStatusCallback);
		}
	},
				
	// UPDATE VIEW METHODS		
	_allowPropertyEdit: function (allowEdits){
		var props = this.ldapConfig._properties;
		for (var i=0; i < props.length; i++) {
			props[i]._editValueNode.disabled = !allowEdits;
		};	 
	},
	
	// POST CONTRIBUTOR UPDATES
	runAction: function(itemId, licenses, caller) {
		var args = {
			itemId: itemId,
			repoOnly: true, 
			jsonLicenses: dojo.toJson(licenses)
		}
		AdminClient.postContributor(args, new SRH(caller, "disableAdminAccess", "postAssignLicensesError"));
	}

});

})();